@@ -16,6 +16,7 @@ use Symfony\Component\DependencyInjection\Argument\AbstractArgument;
 use Symfony\Component\DependencyInjection\Argument\BoundArgument;
 use Symfony\Component\DependencyInjection\Argument\ServiceLocatorArgument;
 use Symfony\Component\DependencyInjection\Argument\TaggedIteratorArgument;
+use Symfony\Component\DependencyInjection\Attribute\Target;
 use Symfony\Component\DependencyInjection\Compiler\AutowireRequiredMethodsPass;
 use Symfony\Component\DependencyInjection\Compiler\DefinitionErrorExceptionPass;
 use Symfony\Component\DependencyInjection\Compiler\ResolveBindingsPass;
@@ -276,4 +277,44 @@ class ResolveBindingsPassTest extends TestCase
 
         $this->assertEquals(['C', 'K'], $definition->getArguments());
     }
+
+    public function testBindingsOnTargetedArguments()
+    {
+        $container = new ContainerBuilder();
+        $container->register('service', TargetedBindingsService::class)
+            ->setAutowired(true)
+            ->setBindings([
+                '$targetName' => 'bound_via_target',
+                '$variableName' => 'bound_via_variable',
+                '$commonName' => 'bound_via_common_name',
+            ]);
+
+        (new ResolveBindingsPass())->process($container);
+
+        $definition = $container->getDefinition('service');
+
+        // 1. Priority: Binding matches the #[Target] name
+        $this->assertSame('bound_via_target', $definition->getArgument(0));
+
+        // 2. Fallback: Binding matches the variable name (Target name 'unusedTarget' has no binding)
+        $this->assertSame('bound_via_variable', $definition->getArgument(1));
+
+        // 3. Equality: Target name and variable name are identical
+        $this->assertSame('bound_via_common_name', $definition->getArgument(2));
+    }
+}
+
+class TargetedBindingsService
+{
+    public function __construct(
+        #[Target('targetName')]
+        public $arg1,
+
+        #[Target('unusedTarget')]
+        public $variableName,
+
+        #[Target('commonName')]
+        public $commonName,
+    ) {
+    }
 }
