@@ -12,8 +12,12 @@
 namespace Symfony\Component\PropertyInfo\Tests\Extractor;
 
 use phpDocumentor\Reflection\DocBlock;
+use phpDocumentor\Reflection\PseudoTypes\IntMask;
+use phpDocumentor\Reflection\PseudoTypes\IntMaskOf;
+use PHPUnit\Framework\Attributes\DataProvider;
+use PHPUnit\Framework\Attributes\Group;
+use PHPUnit\Framework\Attributes\IgnoreDeprecations;
 use PHPUnit\Framework\TestCase;
-use Symfony\Bridge\PhpUnit\ExpectUserDeprecationMessageTrait;
 use Symfony\Component\PropertyInfo\Extractor\PhpDocExtractor;
 use Symfony\Component\PropertyInfo\Tests\Fixtures\ConstructorDummy;
 use Symfony\Component\PropertyInfo\Tests\Fixtures\DockBlockFallback;
@@ -26,6 +30,7 @@ use Symfony\Component\PropertyInfo\Tests\Fixtures\PseudoTypeDummy;
 use Symfony\Component\PropertyInfo\Tests\Fixtures\PseudoTypesDummy;
 use Symfony\Component\PropertyInfo\Tests\Fixtures\TraitUsage\DummyUsedInTrait;
 use Symfony\Component\PropertyInfo\Tests\Fixtures\TraitUsage\DummyUsingTrait;
+use Symfony\Component\PropertyInfo\Tests\Fixtures\VoidNeverReturnTypeDummy;
 use Symfony\Component\PropertyInfo\Type as LegacyType;
 use Symfony\Component\TypeInfo\Type;
 use Symfony\Component\TypeInfo\Type\NullableType;
@@ -35,8 +40,6 @@ use Symfony\Component\TypeInfo\Type\NullableType;
  */
 class PhpDocExtractorTest extends TestCase
 {
-    use ExpectUserDeprecationMessageTrait;
-
     private PhpDocExtractor $extractor;
 
     protected function setUp(): void
@@ -44,11 +47,9 @@ class PhpDocExtractorTest extends TestCase
         $this->extractor = new PhpDocExtractor();
     }
 
-    /**
-     * @group legacy
-     *
-     * @dataProvider provideLegacyTypes
-     */
+    #[IgnoreDeprecations]
+    #[Group('legacy')]
+    #[DataProvider('provideLegacyTypes')]
     public function testExtractLegacy($property, ?array $type, $shortDescription, $longDescription)
     {
         $this->expectUserDeprecationMessage('Since symfony/property-info 7.3: The "Symfony\Component\PropertyInfo\Extractor\PhpDocExtractor::getTypes()" method is deprecated, use "Symfony\Component\PropertyInfo\Extractor\PhpDocExtractor::getType()" instead.');
@@ -71,9 +72,8 @@ class PhpDocExtractorTest extends TestCase
         $this->assertNull($docBlock);
     }
 
-    /**
-     * @group legacy
-     */
+    #[IgnoreDeprecations]
+    #[Group('legacy')]
     public function testParamTagTypeIsOmittedLegacy()
     {
         $this->expectUserDeprecationMessage('Since symfony/property-info 7.3: The "Symfony\Component\PropertyInfo\Extractor\PhpDocExtractor::getTypes()" method is deprecated, use "Symfony\Component\PropertyInfo\Extractor\PhpDocExtractor::getType()" instead.');
@@ -90,11 +90,9 @@ class PhpDocExtractorTest extends TestCase
         ];
     }
 
-    /**
-     * @group legacy
-     *
-     * @dataProvider provideLegacyInvalidTypes
-     */
+    #[IgnoreDeprecations]
+    #[Group('legacy')]
+    #[DataProvider('provideLegacyInvalidTypes')]
     public function testInvalidLegacy($property, $shortDescription, $longDescription)
     {
         $this->expectUserDeprecationMessage('Since symfony/property-info 7.3: The "Symfony\Component\PropertyInfo\Extractor\PhpDocExtractor::getTypes()" method is deprecated, use "Symfony\Component\PropertyInfo\Extractor\PhpDocExtractor::getType()" instead.');
@@ -104,9 +102,8 @@ class PhpDocExtractorTest extends TestCase
         $this->assertSame($longDescription, $this->extractor->getLongDescription('Symfony\Component\PropertyInfo\Tests\Fixtures\InvalidDummy', $property));
     }
 
-    /**
-     * @group legacy
-     */
+    #[IgnoreDeprecations]
+    #[Group('legacy')]
     public function testEmptyParamAnnotationLegacy()
     {
         $this->expectUserDeprecationMessage('Since symfony/property-info 7.3: The "Symfony\Component\PropertyInfo\Extractor\PhpDocExtractor::getTypes()" method is deprecated, use "Symfony\Component\PropertyInfo\Extractor\PhpDocExtractor::getType()" instead.');
@@ -116,11 +113,9 @@ class PhpDocExtractorTest extends TestCase
         $this->assertNull($this->extractor->getLongDescription('Symfony\Component\PropertyInfo\Tests\Fixtures\InvalidDummy', 'foo'));
     }
 
-    /**
-     * @group legacy
-     *
-     * @dataProvider provideLegacyTypesWithNoPrefixes
-     */
+    #[IgnoreDeprecations]
+    #[Group('legacy')]
+    #[DataProvider('provideLegacyTypesWithNoPrefixes')]
     public function testExtractTypesWithNoPrefixesLegacy($property, ?array $type = null)
     {
         $this->expectUserDeprecationMessage('Since symfony/property-info 7.3: The "Symfony\Component\PropertyInfo\Extractor\PhpDocExtractor::getTypes()" method is deprecated, use "Symfony\Component\PropertyInfo\Extractor\PhpDocExtractor::getType()" instead.');
@@ -182,11 +177,9 @@ class PhpDocExtractorTest extends TestCase
         ];
     }
 
-    /**
-     * @group legacy
-     *
-     * @dataProvider provideLegacyCollectionTypes
-     */
+    #[IgnoreDeprecations]
+    #[Group('legacy')]
+    #[DataProvider('provideLegacyCollectionTypes')]
     public function testExtractCollectionLegacy($property, ?array $type, $shortDescription, $longDescription)
     {
         $this->testExtractLegacy($property, $type, $shortDescription, $longDescription);
@@ -246,11 +239,9 @@ class PhpDocExtractorTest extends TestCase
         ];
     }
 
-    /**
-     * @group legacy
-     *
-     * @dataProvider provideLegacyTypesWithCustomPrefixes
-     */
+    #[IgnoreDeprecations]
+    #[Group('legacy')]
+    #[DataProvider('provideLegacyTypesWithCustomPrefixes')]
     public function testExtractTypesWithCustomPrefixesLegacy($property, ?array $type = null)
     {
         $this->expectUserDeprecationMessage('Since symfony/property-info 7.3: The "Symfony\Component\PropertyInfo\Extractor\PhpDocExtractor::getTypes()" method is deprecated, use "Symfony\Component\PropertyInfo\Extractor\PhpDocExtractor::getType()" instead.');
@@ -263,84 +254,80 @@ class PhpDocExtractorTest extends TestCase
     public static function provideLegacyTypesWithCustomPrefixes()
     {
         return [
-            ['foo', null, 'Short description.', 'Long description.'],
-            ['bar', [new LegacyType(LegacyType::BUILTIN_TYPE_STRING)], 'This is bar', null],
-            ['baz', [new LegacyType(LegacyType::BUILTIN_TYPE_INT)], 'Should be used.', null],
-            ['foo2', [new LegacyType(LegacyType::BUILTIN_TYPE_FLOAT)], null, null],
-            ['foo3', [new LegacyType(LegacyType::BUILTIN_TYPE_CALLABLE)], null, null],
-            ['foo4', [new LegacyType(LegacyType::BUILTIN_TYPE_NULL)], null, null],
-            ['foo5', null, null, null],
+            ['foo', null],
+            ['bar', [new LegacyType(LegacyType::BUILTIN_TYPE_STRING)]],
+            ['baz', [new LegacyType(LegacyType::BUILTIN_TYPE_INT)]],
+            ['foo2', [new LegacyType(LegacyType::BUILTIN_TYPE_FLOAT)]],
+            ['foo3', [new LegacyType(LegacyType::BUILTIN_TYPE_CALLABLE)]],
+            ['foo4', [new LegacyType(LegacyType::BUILTIN_TYPE_NULL)]],
+            ['foo5', null],
             [
                 'files',
                 [
                     new LegacyType(LegacyType::BUILTIN_TYPE_ARRAY, false, null, true, new LegacyType(LegacyType::BUILTIN_TYPE_INT), new LegacyType(LegacyType::BUILTIN_TYPE_OBJECT, false, 'SplFileInfo')),
                     new LegacyType(LegacyType::BUILTIN_TYPE_RESOURCE),
                 ],
-                null,
-                null,
             ],
-            ['bal', [new LegacyType(LegacyType::BUILTIN_TYPE_OBJECT, false, 'DateTimeImmutable')], null, null],
-            ['parent', [new LegacyType(LegacyType::BUILTIN_TYPE_OBJECT, false, 'Symfony\Component\PropertyInfo\Tests\Fixtures\ParentDummy')], null, null],
-            ['collection', [new LegacyType(LegacyType::BUILTIN_TYPE_ARRAY, false, null, true, new LegacyType(LegacyType::BUILTIN_TYPE_INT), new LegacyType(LegacyType::BUILTIN_TYPE_OBJECT, false, 'DateTimeImmutable'))], null, null],
-            ['nestedCollection', [new LegacyType(LegacyType::BUILTIN_TYPE_ARRAY, false, null, true, new LegacyType(LegacyType::BUILTIN_TYPE_INT), new LegacyType(LegacyType::BUILTIN_TYPE_ARRAY, false, null, true, new LegacyType(LegacyType::BUILTIN_TYPE_INT), new LegacyType(LegacyType::BUILTIN_TYPE_STRING, false)))], null, null],
-            ['mixedCollection', [new LegacyType(LegacyType::BUILTIN_TYPE_ARRAY, false, null, true, null, null)], null, null],
-            ['a', null, 'A.', null],
-            ['b', null, 'B.', null],
-            ['c', [new LegacyType(LegacyType::BUILTIN_TYPE_BOOL, true)], null, null],
-            ['d', [new LegacyType(LegacyType::BUILTIN_TYPE_BOOL)], null, null],
-            ['e', [new LegacyType(LegacyType::BUILTIN_TYPE_ARRAY, false, null, true, new LegacyType(LegacyType::BUILTIN_TYPE_INT), new LegacyType(LegacyType::BUILTIN_TYPE_RESOURCE))], null, null],
-            ['f', [new LegacyType(LegacyType::BUILTIN_TYPE_ARRAY, false, null, true, new LegacyType(LegacyType::BUILTIN_TYPE_INT), new LegacyType(LegacyType::BUILTIN_TYPE_OBJECT, false, 'DateTimeImmutable'))], null, null],
-            ['g', [new LegacyType(LegacyType::BUILTIN_TYPE_ARRAY, true, null, true)], 'Nullable array.', null],
-            ['h', [new LegacyType(LegacyType::BUILTIN_TYPE_STRING, true)], null, null],
-            ['i', [new LegacyType(LegacyType::BUILTIN_TYPE_STRING, true), new LegacyType(LegacyType::BUILTIN_TYPE_INT, true)], null, null],
-            ['j', [new LegacyType(LegacyType::BUILTIN_TYPE_OBJECT, true, 'DateTimeImmutable')], null, null],
-            ['nullableCollectionOfNonNullableElements', [new LegacyType(LegacyType::BUILTIN_TYPE_ARRAY, true, null, true, new LegacyType(LegacyType::BUILTIN_TYPE_INT), new LegacyType(LegacyType::BUILTIN_TYPE_INT, false))], null, null],
-            ['nonNullableCollectionOfNullableElements', [new LegacyType(LegacyType::BUILTIN_TYPE_ARRAY, false, null, true, new LegacyType(LegacyType::BUILTIN_TYPE_INT), new LegacyType(LegacyType::BUILTIN_TYPE_INT, true))], null, null],
-            ['nullableCollectionOfMultipleNonNullableElementTypes', [new LegacyType(LegacyType::BUILTIN_TYPE_ARRAY, true, null, true, new LegacyType(LegacyType::BUILTIN_TYPE_INT), [new LegacyType(LegacyType::BUILTIN_TYPE_INT), new LegacyType(LegacyType::BUILTIN_TYPE_STRING)])], null, null],
-            ['donotexist', null, null, null],
-            ['staticGetter', null, null, null],
-            ['staticSetter', null, null, null],
+            ['bal', [new LegacyType(LegacyType::BUILTIN_TYPE_OBJECT, false, 'DateTimeImmutable')]],
+            ['parent', [new LegacyType(LegacyType::BUILTIN_TYPE_OBJECT, false, 'Symfony\Component\PropertyInfo\Tests\Fixtures\ParentDummy')]],
+            ['collection', [new LegacyType(LegacyType::BUILTIN_TYPE_ARRAY, false, null, true, new LegacyType(LegacyType::BUILTIN_TYPE_INT), new LegacyType(LegacyType::BUILTIN_TYPE_OBJECT, false, 'DateTimeImmutable'))]],
+            ['nestedCollection', [new LegacyType(LegacyType::BUILTIN_TYPE_ARRAY, false, null, true, new LegacyType(LegacyType::BUILTIN_TYPE_INT), new LegacyType(LegacyType::BUILTIN_TYPE_ARRAY, false, null, true, new LegacyType(LegacyType::BUILTIN_TYPE_INT), new LegacyType(LegacyType::BUILTIN_TYPE_STRING, false)))]],
+            ['mixedCollection', [new LegacyType(LegacyType::BUILTIN_TYPE_ARRAY, false, null, true, null, null)]],
+            ['a', null],
+            ['b', null],
+            ['c', [new LegacyType(LegacyType::BUILTIN_TYPE_BOOL, true)]],
+            ['d', [new LegacyType(LegacyType::BUILTIN_TYPE_BOOL)]],
+            ['e', [new LegacyType(LegacyType::BUILTIN_TYPE_ARRAY, false, null, true, new LegacyType(LegacyType::BUILTIN_TYPE_INT), new LegacyType(LegacyType::BUILTIN_TYPE_RESOURCE))]],
+            ['f', [new LegacyType(LegacyType::BUILTIN_TYPE_ARRAY, false, null, true, new LegacyType(LegacyType::BUILTIN_TYPE_INT), new LegacyType(LegacyType::BUILTIN_TYPE_OBJECT, false, 'DateTimeImmutable'))]],
+            ['g', [new LegacyType(LegacyType::BUILTIN_TYPE_ARRAY, true, null, true)]],
+            ['h', [new LegacyType(LegacyType::BUILTIN_TYPE_STRING, true)]],
+            ['i', [new LegacyType(LegacyType::BUILTIN_TYPE_STRING, true), new LegacyType(LegacyType::BUILTIN_TYPE_INT, true)]],
+            ['j', [new LegacyType(LegacyType::BUILTIN_TYPE_OBJECT, true, 'DateTimeImmutable')]],
+            ['nullableCollectionOfNonNullableElements', [new LegacyType(LegacyType::BUILTIN_TYPE_ARRAY, true, null, true, new LegacyType(LegacyType::BUILTIN_TYPE_INT), new LegacyType(LegacyType::BUILTIN_TYPE_INT, false))]],
+            ['nonNullableCollectionOfNullableElements', [new LegacyType(LegacyType::BUILTIN_TYPE_ARRAY, false, null, true, new LegacyType(LegacyType::BUILTIN_TYPE_INT), new LegacyType(LegacyType::BUILTIN_TYPE_INT, true))]],
+            ['nullableCollectionOfMultipleNonNullableElementTypes', [new LegacyType(LegacyType::BUILTIN_TYPE_ARRAY, true, null, true, new LegacyType(LegacyType::BUILTIN_TYPE_INT), [new LegacyType(LegacyType::BUILTIN_TYPE_INT), new LegacyType(LegacyType::BUILTIN_TYPE_STRING)])]],
+            ['donotexist', null],
+            ['staticGetter', null],
+            ['staticSetter', null],
         ];
     }
 
     public static function provideLegacyTypesWithNoPrefixes()
     {
         return [
-            ['foo', null, 'Short description.', 'Long description.'],
-            ['bar', [new LegacyType(LegacyType::BUILTIN_TYPE_STRING)], 'This is bar', null],
-            ['baz', [new LegacyType(LegacyType::BUILTIN_TYPE_INT)], 'Should be used.', null],
-            ['foo2', [new LegacyType(LegacyType::BUILTIN_TYPE_FLOAT)], null, null],
-            ['foo3', [new LegacyType(LegacyType::BUILTIN_TYPE_CALLABLE)], null, null],
-            ['foo4', [new LegacyType(LegacyType::BUILTIN_TYPE_NULL)], null, null],
-            ['foo5', null, null, null],
+            ['foo', null],
+            ['bar', [new LegacyType(LegacyType::BUILTIN_TYPE_STRING)]],
+            ['baz', [new LegacyType(LegacyType::BUILTIN_TYPE_INT)]],
+            ['foo2', [new LegacyType(LegacyType::BUILTIN_TYPE_FLOAT)]],
+            ['foo3', [new LegacyType(LegacyType::BUILTIN_TYPE_CALLABLE)]],
+            ['foo4', [new LegacyType(LegacyType::BUILTIN_TYPE_NULL)]],
+            ['foo5', null],
             [
                 'files',
                 [
                     new LegacyType(LegacyType::BUILTIN_TYPE_ARRAY, false, null, true, new LegacyType(LegacyType::BUILTIN_TYPE_INT), new LegacyType(LegacyType::BUILTIN_TYPE_OBJECT, false, 'SplFileInfo')),
                     new LegacyType(LegacyType::BUILTIN_TYPE_RESOURCE),
                 ],
-                null,
-                null,
             ],
-            ['bal', [new LegacyType(LegacyType::BUILTIN_TYPE_OBJECT, false, 'DateTimeImmutable')], null, null],
-            ['parent', [new LegacyType(LegacyType::BUILTIN_TYPE_OBJECT, false, 'Symfony\Component\PropertyInfo\Tests\Fixtures\ParentDummy')], null, null],
-            ['collection', [new LegacyType(LegacyType::BUILTIN_TYPE_ARRAY, false, null, true, new LegacyType(LegacyType::BUILTIN_TYPE_INT), new LegacyType(LegacyType::BUILTIN_TYPE_OBJECT, false, 'DateTimeImmutable'))], null, null],
-            ['nestedCollection', [new LegacyType(LegacyType::BUILTIN_TYPE_ARRAY, false, null, true, new LegacyType(LegacyType::BUILTIN_TYPE_INT), new LegacyType(LegacyType::BUILTIN_TYPE_ARRAY, false, null, true, new LegacyType(LegacyType::BUILTIN_TYPE_INT), new LegacyType(LegacyType::BUILTIN_TYPE_STRING, false)))], null, null],
-            ['mixedCollection', [new LegacyType(LegacyType::BUILTIN_TYPE_ARRAY, false, null, true, null, null)], null, null],
-            ['a', null, 'A.', null],
-            ['b', null, 'B.', null],
-            ['c', null, null, null],
-            ['d', null, null, null],
-            ['e', null, null, null],
-            ['f', null, null, null],
-            ['g', [new LegacyType(LegacyType::BUILTIN_TYPE_ARRAY, true, null, true)], 'Nullable array.', null],
-            ['h', [new LegacyType(LegacyType::BUILTIN_TYPE_STRING, true)], null, null],
-            ['i', [new LegacyType(LegacyType::BUILTIN_TYPE_STRING, true), new LegacyType(LegacyType::BUILTIN_TYPE_INT, true)], null, null],
-            ['j', [new LegacyType(LegacyType::BUILTIN_TYPE_OBJECT, true, 'DateTimeImmutable')], null, null],
-            ['nullableCollectionOfNonNullableElements', [new LegacyType(LegacyType::BUILTIN_TYPE_ARRAY, true, null, true, new LegacyType(LegacyType::BUILTIN_TYPE_INT), new LegacyType(LegacyType::BUILTIN_TYPE_INT, false))], null, null],
-            ['donotexist', null, null, null],
-            ['staticGetter', null, null, null],
-            ['staticSetter', null, null, null],
+            ['bal', [new LegacyType(LegacyType::BUILTIN_TYPE_OBJECT, false, 'DateTimeImmutable')]],
+            ['parent', [new LegacyType(LegacyType::BUILTIN_TYPE_OBJECT, false, 'Symfony\Component\PropertyInfo\Tests\Fixtures\ParentDummy')]],
+            ['collection', [new LegacyType(LegacyType::BUILTIN_TYPE_ARRAY, false, null, true, new LegacyType(LegacyType::BUILTIN_TYPE_INT), new LegacyType(LegacyType::BUILTIN_TYPE_OBJECT, false, 'DateTimeImmutable'))]],
+            ['nestedCollection', [new LegacyType(LegacyType::BUILTIN_TYPE_ARRAY, false, null, true, new LegacyType(LegacyType::BUILTIN_TYPE_INT), new LegacyType(LegacyType::BUILTIN_TYPE_ARRAY, false, null, true, new LegacyType(LegacyType::BUILTIN_TYPE_INT), new LegacyType(LegacyType::BUILTIN_TYPE_STRING, false)))]],
+            ['mixedCollection', [new LegacyType(LegacyType::BUILTIN_TYPE_ARRAY, false, null, true, null, null)]],
+            ['a', null],
+            ['b', null],
+            ['c', null],
+            ['d', null],
+            ['e', null],
+            ['f', null],
+            ['g', [new LegacyType(LegacyType::BUILTIN_TYPE_ARRAY, true, null, true)]],
+            ['h', [new LegacyType(LegacyType::BUILTIN_TYPE_STRING, true)]],
+            ['i', [new LegacyType(LegacyType::BUILTIN_TYPE_STRING, true), new LegacyType(LegacyType::BUILTIN_TYPE_INT, true)]],
+            ['j', [new LegacyType(LegacyType::BUILTIN_TYPE_OBJECT, true, 'DateTimeImmutable')]],
+            ['nullableCollectionOfNonNullableElements', [new LegacyType(LegacyType::BUILTIN_TYPE_ARRAY, true, null, true, new LegacyType(LegacyType::BUILTIN_TYPE_INT), new LegacyType(LegacyType::BUILTIN_TYPE_INT, false))]],
+            ['donotexist', null],
+            ['staticGetter', null],
+            ['staticSetter', null],
         ];
     }
 
@@ -364,11 +351,9 @@ class PhpDocExtractorTest extends TestCase
         ];
     }
 
-    /**
-     * @group legacy
-     *
-     * @dataProvider provideLegacyDockBlockFallbackTypes
-     */
+    #[IgnoreDeprecations]
+    #[Group('legacy')]
+    #[DataProvider('provideLegacyDockBlockFallbackTypes')]
     public function testDocBlockFallbackLegacy($property, $types)
     {
         $this->expectUserDeprecationMessage('Since symfony/property-info 7.3: The "Symfony\Component\PropertyInfo\Extractor\PhpDocExtractor::getTypes()" method is deprecated, use "Symfony\Component\PropertyInfo\Extractor\PhpDocExtractor::getType()" instead.');
@@ -376,11 +361,9 @@ class PhpDocExtractorTest extends TestCase
         $this->assertEquals($types, $this->extractor->getTypes('Symfony\Component\PropertyInfo\Tests\Fixtures\DockBlockFallback', $property));
     }
 
-    /**
-     * @group legacy
-     *
-     * @dataProvider provideLegacyPropertiesDefinedByTraits
-     */
+    #[IgnoreDeprecations]
+    #[Group('legacy')]
+    #[DataProvider('provideLegacyPropertiesDefinedByTraits')]
     public function testPropertiesDefinedByTraitsLegacy(string $property, LegacyType $type)
     {
         $this->expectUserDeprecationMessage('Since symfony/property-info 7.3: The "Symfony\Component\PropertyInfo\Extractor\PhpDocExtractor::getTypes()" method is deprecated, use "Symfony\Component\PropertyInfo\Extractor\PhpDocExtractor::getType()" instead.');
@@ -400,11 +383,9 @@ class PhpDocExtractorTest extends TestCase
         ];
     }
 
-    /**
-     * @group legacy
-     *
-     * @dataProvider provideLegacyMethodsDefinedByTraits
-     */
+    #[IgnoreDeprecations]
+    #[Group('legacy')]
+    #[DataProvider('provideLegacyMethodsDefinedByTraits')]
     public function testMethodsDefinedByTraitsLegacy(string $property, LegacyType $type)
     {
         $this->expectUserDeprecationMessage('Since symfony/property-info 7.3: The "Symfony\Component\PropertyInfo\Extractor\PhpDocExtractor::getTypes()" method is deprecated, use "Symfony\Component\PropertyInfo\Extractor\PhpDocExtractor::getType()" instead.');
@@ -424,11 +405,9 @@ class PhpDocExtractorTest extends TestCase
         ];
     }
 
-    /**
-     * @group legacy
-     *
-     * @dataProvider provideLegacyPropertiesStaticType
-     */
+    #[IgnoreDeprecations]
+    #[Group('legacy')]
+    #[DataProvider('provideLegacyPropertiesStaticType')]
     public function testPropertiesStaticTypeLegacy(string $class, string $property, LegacyType $type)
     {
         $this->expectUserDeprecationMessage('Since symfony/property-info 7.3: The "Symfony\Component\PropertyInfo\Extractor\PhpDocExtractor::getTypes()" method is deprecated, use "Symfony\Component\PropertyInfo\Extractor\PhpDocExtractor::getType()" instead.');
@@ -444,11 +423,9 @@ class PhpDocExtractorTest extends TestCase
         ];
     }
 
-    /**
-     * @group legacy
-     *
-     * @dataProvider provideLegacyPropertiesParentType
-     */
+    #[IgnoreDeprecations]
+    #[Group('legacy')]
+    #[DataProvider('provideLegacyPropertiesParentType')]
     public function testPropertiesParentTypeLegacy(string $class, string $property, ?array $types)
     {
         $this->expectUserDeprecationMessage('Since symfony/property-info 7.3: The "Symfony\Component\PropertyInfo\Extractor\PhpDocExtractor::getTypes()" method is deprecated, use "Symfony\Component\PropertyInfo\Extractor\PhpDocExtractor::getType()" instead.');
@@ -464,19 +441,33 @@ class PhpDocExtractorTest extends TestCase
         ];
     }
 
-    /**
-     * @group legacy
-     */
+    #[IgnoreDeprecations]
+    #[Group('legacy')]
     public function testUnknownPseudoTypeLegacy()
     {
         $this->expectUserDeprecationMessage('Since symfony/property-info 7.3: The "Symfony\Component\PropertyInfo\Extractor\PhpDocExtractor::getTypes()" method is deprecated, use "Symfony\Component\PropertyInfo\Extractor\PhpDocExtractor::getType()" instead.');
 
-        $this->assertEquals([new LegacyType(LegacyType::BUILTIN_TYPE_OBJECT, false, 'scalar')], $this->extractor->getTypes(PseudoTypeDummy::class, 'unknownPseudoType'));
+        $this->assertEquals([
+            new LegacyType(LegacyType::BUILTIN_TYPE_OBJECT, false, 'Symfony\\Component\\PropertyInfo\\Tests\\Fixtures\\unknownpseudo')
+        ], $this->extractor->getTypes(PseudoTypeDummy::class, 'unknownPseudoType'));
     }
 
-    /**
-     * @group legacy
-     */
+    #[IgnoreDeprecations]
+    #[Group('legacy')]
+    public function testScalarPseudoTypeLegacy()
+    {
+        $this->expectUserDeprecationMessage('Since symfony/property-info 7.3: The "Symfony\Component\PropertyInfo\Extractor\PhpDocExtractor::getTypes()" method is deprecated, use "Symfony\Component\PropertyInfo\Extractor\PhpDocExtractor::getType()" instead.');
+
+        $this->assertEquals([
+            new LegacyType(LegacyType::BUILTIN_TYPE_BOOL),
+            new LegacyType(LegacyType::BUILTIN_TYPE_FLOAT),
+            new LegacyType(LegacyType::BUILTIN_TYPE_INT),
+            new LegacyType(LegacyType::BUILTIN_TYPE_STRING),
+        ], $this->extractor->getTypes(PseudoTypeDummy::class, 'scalarPseudoType'));
+    }
+
+    #[IgnoreDeprecations]
+    #[Group('legacy')]
     public function testGenericInterface()
     {
         $this->expectUserDeprecationMessage('Since symfony/property-info 7.3: The "Symfony\Component\PropertyInfo\Extractor\PhpDocExtractor::getTypes()" method is deprecated, use "Symfony\Component\PropertyInfo\Extractor\PhpDocExtractor::getType()" instead.');
@@ -484,11 +475,9 @@ class PhpDocExtractorTest extends TestCase
         $this->assertNull($this->extractor->getTypes(Dummy::class, 'genericInterface'));
     }
 
-    /**
-     * @group legacy
-     *
-     * @dataProvider provideLegacyConstructorTypes
-     */
+    #[IgnoreDeprecations]
+    #[Group('legacy')]
+    #[DataProvider('provideLegacyConstructorTypes')]
     public function testExtractConstructorTypesLegacy($property, ?array $type = null)
     {
         $this->expectUserDeprecationMessage('Since symfony/property-info 7.3: The "Symfony\Component\PropertyInfo\Extractor\PhpDocExtractor::getTypesFromConstructor()" method is deprecated, use "Symfony\Component\PropertyInfo\Extractor\PhpDocExtractor::getTypeFromConstructor()" instead.');
@@ -508,11 +497,9 @@ class PhpDocExtractorTest extends TestCase
         ];
     }
 
-    /**
-     * @group legacy
-     *
-     * @dataProvider provideLegacyPseudoTypes
-     */
+    #[IgnoreDeprecations]
+    #[Group('legacy')]
+    #[DataProvider('provideLegacyPseudoTypes')]
     public function testPseudoTypesLegacy($property, array $type)
     {
         $this->expectUserDeprecationMessage('Since symfony/property-info 7.3: The "Symfony\Component\PropertyInfo\Extractor\PhpDocExtractor::getTypes()" method is deprecated, use "Symfony\Component\PropertyInfo\Extractor\PhpDocExtractor::getType()" instead.');
@@ -535,11 +522,9 @@ class PhpDocExtractorTest extends TestCase
         ];
     }
 
-    /**
-     * @group legacy
-     *
-     * @dataProvider provideLegacyPromotedProperty
-     */
+    #[IgnoreDeprecations]
+    #[Group('legacy')]
+    #[DataProvider('provideLegacyPromotedProperty')]
     public function testExtractPromotedPropertyLegacy(string $property, ?array $types)
     {
         $this->expectUserDeprecationMessage('Since symfony/property-info 7.3: The "Symfony\Component\PropertyInfo\Extractor\PhpDocExtractor::getTypes()" method is deprecated, use "Symfony\Component\PropertyInfo\Extractor\PhpDocExtractor::getType()" instead.');
@@ -560,9 +545,7 @@ class PhpDocExtractorTest extends TestCase
         $this->assertNull($this->extractor->getType(OmittedParamTagTypeDocBlock::class, 'omittedType'));
     }
 
-    /**
-     * @dataProvider typeProvider
-     */
+    #[DataProvider('typeProvider')]
     public function testExtract(string $property, ?Type $type, ?string $shortDescription, ?string $longDescription)
     {
         $this->assertEquals($type, $this->extractor->getType(Dummy::class, $property));
@@ -622,9 +605,7 @@ class PhpDocExtractorTest extends TestCase
         yield ['collectionAsObject', Type::collection(Type::object(DummyCollection::class), Type::string(), Type::int()), null, null];
     }
 
-    /**
-     * @dataProvider invalidTypeProvider
-     */
+    #[DataProvider('invalidTypeProvider')]
     public function testInvalid(string $property, ?string $shortDescription, ?string $longDescription)
     {
         $this->assertNull($this->extractor->getType(InvalidDummy::class, $property));
@@ -642,9 +623,7 @@ class PhpDocExtractorTest extends TestCase
         yield 'bar' => ['bar', 'Bar.', null];
     }
 
-    /**
-     * @dataProvider typeWithNoPrefixesProvider
-     */
+    #[DataProvider('typeWithNoPrefixesProvider')]
     public function testExtractTypesWithNoPrefixes(string $property, ?Type $type)
     {
         $noPrefixExtractor = new PhpDocExtractor(null, [], [], []);
@@ -690,9 +669,7 @@ class PhpDocExtractorTest extends TestCase
         yield ['staticSetter', null];
     }
 
-    /**
-     * @dataProvider provideCollectionTypes
-     */
+    #[DataProvider('provideCollectionTypes')]
     public function testExtractCollection(string $property, ?Type $type)
     {
         $this->testExtract($property, $type, null, null);
@@ -706,13 +683,11 @@ class PhpDocExtractorTest extends TestCase
         yield ['iteratorCollection', Type::collection(Type::object(\Iterator::class), Type::string())];
         yield ['iteratorCollectionWithKey', Type::collection(Type::object(\Iterator::class), Type::string(), Type::int())];
         yield ['nestedIterators', Type::collection(Type::object(\Iterator::class), Type::collection(Type::object(\Iterator::class), Type::string(), Type::int()), Type::int())];
-        yield ['arrayWithKeys', Type::dict(Type::string()), null, null];
-        yield ['arrayWithKeysAndComplexValue', Type::dict(Type::nullable(Type::array(Type::nullable(Type::string()), Type::int()))), null, null];
+        yield ['arrayWithKeys', Type::dict(Type::string())];
+        yield ['arrayWithKeysAndComplexValue', Type::dict(Type::nullable(Type::array(Type::nullable(Type::string()), Type::int())))];
     }
 
-    /**
-     * @dataProvider typeWithCustomPrefixesProvider
-     */
+    #[DataProvider('typeWithCustomPrefixesProvider')]
     public function testExtractTypeWithCustomPrefixes(string $property, ?Type $type)
     {
         $customExtractor = new PhpDocExtractor(null, ['add', 'remove'], ['is', 'can']);
@@ -763,9 +738,7 @@ class PhpDocExtractorTest extends TestCase
         yield ['staticSetter', null];
     }
 
-    /**
-     * @dataProvider dockBlockFallbackTypesProvider
-     */
+    #[DataProvider('dockBlockFallbackTypesProvider')]
     public function testDocBlockFallback(string $property, ?Type $type)
     {
         $this->assertEquals($type, $this->extractor->getType(DockBlockFallback::class, $property));
@@ -781,9 +754,7 @@ class PhpDocExtractorTest extends TestCase
         yield ['protMut', Type::bool()];
     }
 
-    /**
-     * @dataProvider propertiesDefinedByTraitsProvider
-     */
+    #[DataProvider('propertiesDefinedByTraitsProvider')]
     public function testPropertiesDefinedByTraits(string $property, ?Type $type)
     {
         $this->assertEquals($type, $this->extractor->getType(DummyUsingTrait::class, $property));
@@ -802,9 +773,7 @@ class PhpDocExtractorTest extends TestCase
         yield ['propertyInExternalTraitObjectDifferentNamespace', Type::object(DummyUsedInTrait::class)];
     }
 
-    /**
-     * @dataProvider methodsDefinedByTraitsProvider
-     */
+    #[DataProvider('methodsDefinedByTraitsProvider')]
     public function testMethodsDefinedByTraits(string $property, ?Type $type)
     {
         $this->assertEquals($type, $this->extractor->getType(DummyUsingTrait::class, $property));
@@ -825,9 +794,8 @@ class PhpDocExtractorTest extends TestCase
 
     /**
      * @param class-string $class
-     *
-     * @dataProvider propertiesStaticTypeProvider
      */
+    #[DataProvider('propertiesStaticTypeProvider')]
     public function testPropertiesStaticType(string $class, string $property, ?Type $type)
     {
         $this->assertEquals($type, $this->extractor->getType($class, $property));
@@ -844,9 +812,8 @@ class PhpDocExtractorTest extends TestCase
 
     /**
      * @param class-string $class
-     *
-     * @dataProvider propertiesParentTypeProvider
      */
+    #[DataProvider('propertiesParentTypeProvider')]
     public function testPropertiesParentType(string $class, string $property, ?Type $type)
     {
         $this->assertEquals($type, $this->extractor->getType($class, $property));
@@ -863,12 +830,16 @@ class PhpDocExtractorTest extends TestCase
 
     public function testUnknownPseudoType()
     {
-        $this->assertEquals(Type::object('scalar'), $this->extractor->getType(PseudoTypeDummy::class, 'unknownPseudoType'));
+        $this->assertEquals(Type::object('Symfony\\Component\\PropertyInfo\\Tests\\Fixtures\\unknownpseudo'), $this->extractor->getType(PseudoTypeDummy::class, 'unknownPseudoType'));
     }
 
-    /**
-     * @dataProvider constructorTypesProvider
-     */
+    public function testScalarPseudoType()
+    {
+        $this->assertEquals(Type::object('scalar'), $this->extractor->getType(PseudoTypeDummy::class, 'scalarPseudoType'));
+    }
+
+
+    #[DataProvider('constructorTypesProvider')]
     public function testExtractConstructorType(string $property, ?Type $type)
     {
         $this->assertEquals($type, $this->extractor->getTypeFromConstructor(ConstructorDummy::class, $property));
@@ -887,9 +858,7 @@ class PhpDocExtractorTest extends TestCase
         yield ['mixed', Type::mixed()];
     }
 
-    /**
-     * @dataProvider pseudoTypeProvider
-     */
+    #[DataProvider('pseudoTypeProvider')]
     public function testPseudoType(string $property, ?Type $type)
     {
         $this->assertEquals($type, $this->extractor->getType(PseudoTypesDummy::class, $property));
@@ -909,11 +878,24 @@ class PhpDocExtractorTest extends TestCase
         yield ['numericString', Type::string()];
         yield ['traitString', Type::string()];
         yield ['positiveInt', Type::int()];
+        yield ['true', Type::true()];
+        yield ['false', Type::false()];
+        yield ['valueOfStrings', null];
+        yield ['valueOfIntegers', null];
+        yield ['valueOfIntEnum', null];
+        yield ['valueOfStringEnum', null];
+        yield ['valueOfNullableIntEnum', null];
+        yield ['valueOfNullableStringEnum', null];
+        yield ['keyOfStrings', null];
+        yield ['keyOfIntegers', null];
+        yield ['arrayKey', null];
+        yield ['intMask', class_exists(IntMask::class) ? Type::int() : null];
+        yield ['intMaskOf', class_exists(IntMaskOf::class) ? Type::int() : null];
+        yield ['conditional', null];
+        yield ['offsetAccess', null];
     }
 
-    /**
-     * @dataProvider promotedPropertyProvider
-     */
+    #[DataProvider('promotedPropertyProvider')]
     public function testExtractPromotedProperty(string $property, ?Type $type)
     {
         $this->assertEquals($type, $this->extractor->getType(Php80Dummy::class, $property));
@@ -927,6 +909,29 @@ class PhpDocExtractorTest extends TestCase
         yield ['promoted', null];
         yield ['promotedAndMutated', Type::string()];
     }
+
+    public function testSkipVoidNeverReturnTypeAccessors()
+    {
+        // Methods that return void or never should be skipped, so no types should be extracted
+        $this->assertNull($this->extractor->getType(VoidNeverReturnTypeDummy::class, 'voidProperty'));
+        $this->assertNull($this->extractor->getType(VoidNeverReturnTypeDummy::class, 'neverProperty'));
+        // Normal getter should still work
+        $this->assertEquals(Type::string(), $this->extractor->getType(VoidNeverReturnTypeDummy::class, 'normalProperty'));
+    }
+
+    #[IgnoreDeprecations]
+    #[Group('legacy')]
+    public function testSkipVoidNeverReturnTypeAccessorsLegacy()
+    {
+        // Methods that return void or never should be skipped, so no types should be extracted
+        $this->assertNull($this->extractor->getTypes(VoidNeverReturnTypeDummy::class, 'voidProperty'));
+        $this->assertNull($this->extractor->getTypes(VoidNeverReturnTypeDummy::class, 'neverProperty'));
+        // Normal getter should still work
+        $types = $this->extractor->getTypes(VoidNeverReturnTypeDummy::class, 'normalProperty');
+        $this->assertNotNull($types);
+        $this->assertCount(1, $types);
+        $this->assertEquals(LegacyType::BUILTIN_TYPE_STRING, $types[0]->getBuiltinType());
+    }
 }
 
 class EmptyDocBlock
